@@ -213,7 +213,7 @@ class Client(object):
 
     def do_query(self, query=None, qid=None, columns=None, sort=[3],
                  num=None, skip=None, ascending=True,
-                 database=None, round_ints=False):
+                 database=None, round_ints=False, require_all=False):
         """Perform query and return results (list of dicts)."""
 
         if len([q for q in (query, qid) if q]) != 1:
@@ -230,10 +230,7 @@ class Client(object):
                     "sortBy": self.get_sort_list(sort, ascending)
                 }
 
-
-                response =  self.perform_query(body=body)
-
-                data = self.fix_nums(response.get('data'))
+                data = self.get_data(body, require_all=require_all, skip=skip, top=num)
 
                 if round_ints:
                     data = self.round_ints(data)
@@ -247,10 +244,8 @@ class Client(object):
             try:
                 url = f"https://api.quickbase.com/v1/reports/{qid}/run"
 
-                response = self.run_report(
-                    url, database=database, skip=skip, max=num)
-
-                data = self.fix_nums(response.get('data'))
+                data = self.get_data(url=url, database=database, require_all=require_all, 
+                                     skip=skip, top=num, report=True)
 
                 if round_ints:
                     data = self.round_ints(data)
@@ -289,9 +284,7 @@ class Client(object):
                 if groupBy:
                     body.update({"groupBy": groupBy})
 
-                response = self.perform_query(body=body)
-
-                data = self.fix_nums(response.get('data'))
+                data = self.get_data(body, require_all=require_all, skip=skip, top=num)
 
                 if round_ints:
                     data = self.round_ints(data)
@@ -302,12 +295,86 @@ class Client(object):
             except Exception as e:
                 print(traceback.format_exc())
 
-    def run_report(self, url=None, database=None, skip=None, max=None):
+    def get_data(self, body=None, url=None, database=None, 
+                 require_all=False, skip=None, top=None, report=False):
+        '''
+        Intermediate function to separate parsing logic from do_query
+        Calls one of the api functions run_report or perform_query,
+        then returns the data after parsing. 
+        Will loop through multiple api calls for a single query if 
+        the response does not contain all records and require_all is True
+        '''
+        # default offset and goal are set to skip and top
+        offset = skip if skip else 0
+        goal = top if top else 0
+
+        # get response
+        if report:
+            response = self.run_report(url, database=database, skip=offset, top=top)
+        else:
+            
+            body.update({'options': {
+                'skip': offset,
+                'top': goal
+            }})
+
+            response = self.perform_query(body=body)
+
+        # get the number of records received and the total number
+        metadata = response.get('metadata', {})
+        num_records = metadata.get('numRecords', 0)
+        total_records = metadata.get('totalRecords', 0)
+        
+        # if goal is not already zero and not greater than the possible number of records, 
+        # subtract the received records from the goal
+        if goal and goal <= total_records - offset:
+            goal -= num_records
+        # otherwise reset goal as total records - received records - skipped records
+        else:
+            goal = total_records - num_records - offset
+        
+        # add received records to offset number so they get skipped later
+        offset += num_records
+
+        # set data to received records
+        data = self.fix_nums(response.get('data'))
+
+        # if all records are required:
+        if require_all:
+            # loop until goal reaches 0
+            while goal > 0:
+                # get response
+                if report:
+                    response = self.run_report(url, database=database, skip=offset, top=goal)
+                else:
+                    
+                    body.update({'options': {
+                        'skip': offset,
+                        'top': goal
+                    }})
+
+                    response = self.perform_query(body=body)
+
+                # get number of received records
+                metadata = response.get('metadata', {})
+                num_records = metadata.get('numRecords', 0)
+
+                # add received records to offset so they're skipped in following queries
+                # and subtract received records so that the goal approaches 0
+                offset += num_records
+                goal -= num_records
+
+                # add received records to data
+                data.extend(self.fix_nums(response.get('data')))
+
+        return data
+
+    def run_report(self, url=None, database=None, skip=None, top=None):
         """Uses the QB endpoint to actually get resulting report data from QB"""
         response = requests.post(url, params={
             'tableId': database,
             'skip': skip,
-            'top': max
+            'top': top
         }, headers=self.headers)
 
         return response.json()
