@@ -1,5 +1,5 @@
 import time
-import json_quickbase, xml_quickbase
+from . import json_quickbase, xml_quickbase
 from typing import Union, Dict
 import os
 import getpass
