@@ -6,12 +6,11 @@ http://www.quickbase.com/api-guide/index.html
 """
 import os
 import base64
-import datetime
 
 try:
     from lxml import etree, html
 except ImportError:
-    print("You must install lxml and libxml2 packages before using pyQuickBase")
+    print("You must install lxml and libxml2 packages before using this sdk")
 
 import requests
 from requests.exceptions import ConnectionError as reqConnectionError
